// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       mgl_ui_event_defs.h
 *  \brief      MGL UIイベント関連定義
 *  \date       Since: July 7, 2023. 11:06:27 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#ifndef INCGUARD_MGL_UI_EVENT_DEFS_H_1688695587
#define INCGUARD_MGL_UI_EVENT_DEFS_H_1688695587

#include <mgl/math/mgl_rectangle.h>
#include <mgl/input/touch/mgl_touch_state.h>

namespace MGL::UI
{
class Widget;

//! イベントタイプ
enum class EventType
{
    None,               //!< なし
    Focus,              //!< フォーカス
    Unfocus,            //!< フォーカス解除
    Decide,             //!< 決定
    DecideAndUnfocus,   //!< 決定後にフォーカス解除
};

//! イベントステート
enum class EventState
{
    None,       //!< 状態なし
    Focusing,   //!< フォーカス中
};

using EventID = int32_t;
inline constexpr const EventID kInvalidEventID = -1;

struct EventTarget
{
    Widget *widget;
    Rectangle rectangle;
    
    //! コンストラクタ
    constexpr EventTarget(Widget *widget_, const Rectangle &rectangle_) noexcept
        : widget(widget_)
        , rectangle(rectangle_)
    {
    }
};

struct EventResult
{
    EventType       type;
    EventID identifier;
    uint32_t        argument;
    Input::TouchID  touchID;

    constexpr EventResult() noexcept
        : type(EventType::None)
        , identifier(kInvalidEventID)
        , argument(0)
        , touchID(MGL::Input::TouchID::Invalid)
    {
    }

    constexpr EventResult(
            EventType type_,
            EventID identifier_,
            uint32_t argument_ = 0,
            Input::TouchID touchID_ = MGL::Input::TouchID::Invalid) noexcept
        : type(type_)
        , identifier(identifier_)
        , argument(argument_)
        , touchID(touchID_)
    {
    }
};

}   // namespace MGL::UI
#endif	// INCGUARD_MGL_UI_EVENT_DEFS_H_1688695587

// vim: et ts=4 sw=4 sts=4
